/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


////////////////////////////////////////////////////////////
//
// Implementation of Netscape entry points (NPN_*)
//
#include "npplat.h"

extern NPNetscapeFuncs NPNFuncs;

void NPN_Version(int* plugin_major, int* plugin_minor, int* netscape_major, int* netscape_minor)
{
  *plugin_major   = NP_VERSION_MAJOR;
  *plugin_minor   = NP_VERSION_MINOR;
  *netscape_major = HIBYTE(NPNFuncs.version);
  *netscape_minor = LOBYTE(NPNFuncs.version);
}

NPError NPN_GetURLNotify(NPP instance, const char *url, const char *target, void* notifyData)
{
	int navMinorVers = NPNFuncs.version & 0xFF;
  NPError rv = NPERR_NO_ERROR;

  if( navMinorVers >= NPVERS_HAS_NOTIFICATION )
		rv = CallNPN_GetURLNotifyProc(NPNFuncs.geturlnotify, instance, url, target, notifyData);
	else
		rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

NPError NPN_GetURL(NPP instance, const char *url, const char *target)
{
  NPError rv = CallNPN_GetURLProc(NPNFuncs.geturl, instance, url, target);
  return rv;
}

NPError NPN_PostURLNotify(NPP instance, const char* url, const char* window, uint32 len, const char* buf, NPBool file, void* notifyData)
{
	int navMinorVers = NPNFuncs.version & 0xFF;
  NPError rv = NPERR_NO_ERROR;

	if( navMinorVers >= NPVERS_HAS_NOTIFICATION )
		rv = CallNPN_PostURLNotifyProc(NPNFuncs.posturlnotify, instance, url, window, len, buf, file, notifyData);
	else
		rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

NPError NPN_PostURL(NPP instance, const char* url, const char* window, uint32 len, const char* buf, NPBool file)
{
  NPError rv = CallNPN_PostURLProc(NPNFuncs.posturl, instance, url, window, len, buf, file);
  return rv;
} 

NPError NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
{
  NPError rv = CallNPN_RequestReadProc(NPNFuncs.requestread, stream, rangeList);
  return rv;
}

NPError NPN_NewStream(NPP instance, NPMIMEType type, const char* target, NPStream** stream)
{
	int navMinorVersion = NPNFuncs.version & 0xFF;

  NPError rv = NPERR_NO_ERROR;

	if( navMinorVersion >= NPVERS_HAS_STREAMOUTPUT )
		rv = CallNPN_NewStreamProc(NPNFuncs.newstream, instance, type, target, stream);
	else
		rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

int32 NPN_Write(NPP instance, NPStream *stream, int32 len, void *buffer)
{
	int navMinorVersion = NPNFuncs.version & 0xFF;
  int32 rv = 0;

  if( navMinorVersion >= NPVERS_HAS_STREAMOUTPUT )
		rv = CallNPN_WriteProc(NPNFuncs.write, instance, stream, len, buffer);
	else
		rv = -1;

  return rv;
}

NPError NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
{
	int navMinorVersion = NPNFuncs.version & 0xFF;
  NPError rv = NPERR_NO_ERROR;

  if( navMinorVersion >= NPVERS_HAS_STREAMOUTPUT )
		rv = CallNPN_DestroyStreamProc(NPNFuncs.destroystream, instance, stream, reason);
	else
		rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

void NPN_Status(NPP instance, const char *message)
{
  CallNPN_StatusProc(NPNFuncs.status, instance, message);
}

const char* NPN_UserAgent(NPP instance)
{
  const char * rv = NULL;
  rv = CallNPN_UserAgentProc(NPNFuncs.uagent, instance);
  return rv;
}

void* NPN_MemAlloc(uint32 size)
{
  void * rv = NULL;
  rv = CallNPN_MemAllocProc(NPNFuncs.memalloc, size);
  return rv;
}

void NPN_MemFree(void* ptr)
{
  CallNPN_MemFreeProc(NPNFuncs.memfree, ptr);
}

uint32 NPN_MemFlush(uint32 size)
{
  uint32 rv = CallNPN_MemFlushProc(NPNFuncs.memflush, size);
  return rv;
}

void NPN_ReloadPlugins(NPBool reloadPages)
{
  CallNPN_ReloadPluginsProc(NPNFuncs.reloadplugins, reloadPages);
}

#ifdef OJI
JRIEnv* NPN_GetJavaEnv(void)
{
  JRIEnv * rv = NULL;
	rv = CallNPN_GetJavaEnvProc(NPNFuncs.getJavaEnv);
  return rv;
}

jref NPN_GetJavaPeer(NPP instance)
{
  jref rv;
  rv = CallNPN_GetJavaPeerProc(NPNFuncs.getJavaPeer, instance);
  return rv;
}
#endif

NPError NPN_GetValue(NPP instance, NPNVariable variable, void *value)
{
  NPError rv = CallNPN_GetValueProc(NPNFuncs.getvalue, instance, variable, value);
  return rv;
}

NPError NPN_SetValue(NPP instance, NPPVariable variable, void *value)
{
  NPError rv = CallNPN_SetValueProc(NPNFuncs.setvalue, instance, variable, value);
  return rv;
}

void NPN_InvalidateRect(NPP instance, NPRect *invalidRect)
{
  CallNPN_InvalidateRectProc(NPNFuncs.invalidaterect, instance, invalidRect);
}

void NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion)
{
  CallNPN_InvalidateRegionProc(NPNFuncs.invalidateregion, instance, invalidRegion);
}

void NPN_ForceRedraw(NPP instance)
{
  CallNPN_ForceRedrawProc(NPNFuncs.forceredraw, instance);
}

NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *name)
{
  NPIdentifier identifier = NULL;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {
    identifier = NPNFuncs.getstringidentifier(name);
  }
  return identifier;
}

void NPN_GetStringIdentifiers(const NPUTF8 **names, uint32_t nameCount,
                              NPIdentifier *identifiers)
{
  if(LOBYTE(NPNFuncs.version) >= 14)
  {
    NPNFuncs.getstringidentifiers(names, nameCount, identifiers);
  }
}

NPIdentifier NPN_GetStringIdentifier(int32_t intid)
{
  NPIdentifier identifier = NULL;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {
    identifier = NPNFuncs.getintidentifier(intid);
  }
  return identifier;
}

bool NPN_IdentifierIsString(NPIdentifier identifier)
{
  bool isstring = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    isstring = NPNFuncs.identifierisstring(identifier);
  }
  return isstring;
}

NPUTF8 *NPN_UTF8FromIdentifier(NPIdentifier identifier)
{
  NPUTF8 *utf8 = NULL;  
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    utf8 = NPNFuncs.utf8fromidentifier(identifier);
  }
  return utf8;
}

int32_t NPN_IntFromIdentifier(NPIdentifier identifier)
{
  int32_t i = 0;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    i = NPNFuncs.intfromidentifier(identifier);
  }
  return i;
}

NPObject *NPN_CreateObject(NPP npp, NPClass *aClass)
{
  NPObject *object = NULL;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    object = NPNFuncs.createobject(npp, aClass);
  }
  return object;
}

NPObject *NPN_RetainObject(NPObject *obj)
{
  NPObject *object = NULL;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    object = NPNFuncs.retainobject(obj);
  }
  return object;
}

void NPN_ReleaseObject(NPObject *obj)
{
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    NPNFuncs.releaseobject(obj);
  }
}

bool NPN_Invoke(NPP npp, NPObject* obj, NPIdentifier methodName,
                const NPVariant *args, uint32_t argCount, NPVariant *result)
{
  bool ret = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {  
    ret = NPNFuncs.invoke(npp, obj, methodName, args, argCount, result);
  }
  return ret;
}

bool NPN_InvokeDefault(NPP npp, NPObject* obj, const NPVariant *args,
                       uint32_t argCount, NPVariant *result)
{
  bool ret = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    ret = NPNFuncs.invokeDefault(npp, obj, args, argCount, result);
  }
  return ret;
}

bool NPN_Evaluate(NPP npp, NPObject* obj, NPString *script,
                  NPVariant *result)
{
  bool ret = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    ret = NPNFuncs.evaluate(npp, obj, script, result);
  }
  return ret;
}

bool NPN_GetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
                     NPVariant *result)
{
  bool ret = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    ret = NPNFuncs.getproperty(npp, obj, propertyName, result);
  }
  return ret;
}

bool NPN_SetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
                     const NPVariant *value)
{
  bool result = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    result = NPNFuncs.setproperty(npp, obj, propertyName, value);
  }
  return result;
}

bool NPN_RemoveProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
{
  bool result = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    result = NPNFuncs.removeproperty(npp, obj, propertyName);
  }
  return result;
}

bool NPN_HasProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
{
  bool result = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    result = NPNFuncs.hasproperty(npp, obj, propertyName);
  }
  return result;
}

bool NPN_HasMethod(NPP npp, NPObject* obj, NPIdentifier methodName)
{
  bool result = false;
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    result = NPNFuncs.hasmethod(npp, obj, methodName);
  }
  return result;
}

void NPN_ReleaseVariantValue(NPVariant *variant)
{
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    NPNFuncs.releasevariantvalue(variant);
  }
}

void NPN_SetException(NPObject* obj, const NPUTF8 *message)
{
  if(LOBYTE(NPNFuncs.version) >= 14)
  {    
    NPNFuncs.setexception(obj, message);
  }
}
